<?php

namespace Modules\Cotacao\Modules\Pedido\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Modules\Cotacao\Modules\Pedido\Services\Sup09600IImportacaodeRFPService;

class Sup09600IImportacaodeRFPController
{
    protected $newCode;
    protected $environment;
    protected $result;
    protected $errors = array();
    protected $errorMessages = array();
    protected $reportMapping = array ();

    public function __construct($newCode, $environment, Sup09600IImportacaodeRFPService $service)
    {
        $this->newCode       = $newCode;
        $this->environment   = $environment;
        $this->service       = $service;
    }

    public function importarXlsRFB(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $table = $this->service->xlsReadInsert($row, $request);
            $response->addDataSet(new DataSet('data', $table));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function checkItens(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $nrRFPMain = $this->service->searchNRRFPMAIN($row);
            $this->service->execCheckImportProcedure($nrRFPMain);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteImport(DTO\Request $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getRow();
            $params = array(
                "NRRFPMAIN" => $conditions["NRRFPMAIN"]
            );

            $data = $this->service->checkStatusRFP($conditions["NRRFPMAIN"]);

            if (isset($data) && count($data)) {
                if ($data[0]['STATUS'] === "GERADO") {
                    throw new \Exception("IMPOSSIVEL_EXCLUIR_GERADO");
                }
            }

            $this->service->deleteRFPITEMCOTA($params["NRRFPMAIN"]);
            $this->service->deleteRFPFORNECEDOR($params["NRRFPMAIN"]);
            $this->service->deleteRFPREGIAO($params["NRRFPMAIN"]);
            $this->service->deleteRFPPRODMARCA($params["NRRFPMAIN"]);
            $this->service->deleteRFMAIN($params["NRRFPMAIN"]);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    public function gerarPedido(DTO\Request $request, DTO\Response $response)
    {
        $conditions = $request->getRow();

        $params = array(
            "NRRFPMAIN" => $conditions["NRRFPMAIN"],
            "IDDISMONTLIS" => $conditions["IDDISMONTLIS"]
        );
        try {
            $result = $this->service->execImportaRFPProcedure($params);

            if (isset($result['ERROR'])) {
                throw new \Exception($result['ERROR']);
            }

            if (isset($result['MESSAGE'])) {
                $response->addMessage(new Message($result['MESSAGE']));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $this->stNomeRel = $this->getReportName($row);
        $row['NRORG'] = $this->environment->getNrOrg();
        $this->reportMapping[$this->stNomeRel] =
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'NMRAZSOCFORN'             => 'P_NMRAZSOCFORN',
                'CDREGIAO'                 => 'P_CDREGIAO',
                'CDCOMPRADOR'              => 'P_CDCOMPRADOR',
                'DT_VALIDADE_IMPORTACAO'   => 'P_DTIMPORTACAO',
                'DT_VALIDADE'              => 'P_DTVALIDADE',
                'NRORG'                    => 'P_NRORG'    
            ),
        );
  
    }
}
